import os
import logging
import asyncio
import importlib

from pyrogram import filters, enums
from pyrogram.handlers import MessageHandler
from pyrogram.errors.exceptions.bad_request_400 import MessageIdInvalid


class OnCmd:
    @classmethod
    def on_cmd(
        self,
        command: list,
        owner: bool = False,
        group: bool = False,
        userbot: bool = False
    ):
        """
        ## Main decorator

        ### Arguments:

            ``command``: List of commands
            ``owner`` (optional): True if the command is only for owner (Defaults to False)
            ``group`` (optional): True if the command is only for group chats (Defaults to False)
        """
        if userbot:
            if owner:
                _filter = (filters.user(self.OWNER_ID) & filters.me & filters.command(command, self.PREFIX) & ~filters.via_bot & ~filters.forwarded)
            else:
                _filter = (filters.me & filters.command(command, self.PREFIX) & ~filters.via_bot & ~filters.forwarded)
        else:
            if owner:
                _filter = (filters.user(self.OWNER_ID) & filters.command(command) & ~filters.via_bot & ~filters.forwarded)
            else:
                _filter = (filters.command(command) & ~filters.via_bot & ~filters.forwarded)
        def decorators(func):
            async def wrapper(client, message):
                chat_type = message.chat.type
                if group and chat_type == enums.ChatType.PRIVATE:
                    return await message.reply_text(text="`Yo, perintah ini hanya untuk Group!`")
                try:
                    await func(client, message)
                except MessageIdInvalid:
                    logging.warning("Jangan hapus pesan saat memproses. Ini dapat merusak bot!")
                except BaseException as e:
                    logging.error(f"\nModule - {func.__module__} | Command: {command[0]} | Traceback: \n{e}")
                    
            self.add_handler(wrapper, _filter)
            return wrapper
        return decorators 
    
    
    @classmethod
    def on_cf(self, custom_filters):
        """
        ## Decorator to handle custom filters

        ### Arguments:

            ``custom_filters``: Custom filters to handle
        """
        def decorators(func):
            async def wrapper(client, message):
                try:
                    await func(client, message)
                except MessageIdInvalid:
                    logging.warning("Jangan hapus pesan saat memproses. Ini dapat merusak bot!")
                except BaseException as e:
                    logging.error(f"\nModule - {func.__module__} | Command: (Noting, Custom Filter)")
                message.continue_propagation()
            self.add_handler(wrapper, custom_filters)
            return wrapper
        return decorators 
    
    @classmethod
    def add_handler(self, wrapper, _filter):
        """
        ## Add handler to the bot & userbot

        ### Arguments:

            ``wrapper``: Callback function
            ``_filter``: Filters
        """
        self.add_handler(MessageHandler(wrapper, filters=_filter))